#ifndef __GBASIC_GOBJCRUNTIME_H__
#define __GBASIC_GOBJCRUNTIME_H__
/**********************************************************************
  GBasic/GObjCRuntime.h
**********************************************************************/
/*-
 * Copyright (c) 2004  Goichi Hirakawa  <gooichi AT gyazsquare DOT com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

//
//  Platform specific defs for externs
//

//
// For MACH
//

#if defined(__MACH__)

#ifdef __cplusplus
#define GBASIC_EXTERN		extern "C"
#define GBASIC_PRIVATE_EXTERN	__private_extern__
#else
#define GBASIC_EXTERN		extern
#define GBASIC_PRIVATE_EXTERN	__private_extern__
#endif

//
// For Windows
//

#elif defined(WIN32)

#ifndef _BUILDING_GBASIC_DLL
#define _GBASIC_DLL_GOOP	__declspec(dllimport)
#else
#define _GBASIC_DLL_GOOP	__declspec(dllexport)
#endif

#ifdef __cplusplus
#define GBASIC_EXTERN		extern "C" _GBASIC_DLL_GOOP
#define GBASIC_PRIVATE_EXTERN	extern
#else
#define GBASIC_EXTERN		_GBASIC_DLL_GOOP extern
#define GBASIC_PRIVATE_EXTERN	extern
#endif

//
//  For Solaris
//

#elif defined(SOLARIS)

#ifdef __cplusplus
#define GBASIC_EXTERN		extern "C"
#define GBASIC_PRIVATE_EXTERN	extern "C"
#else
#define GBASIC_EXTERN		extern
#define GBASIC_PRIVATE_EXTERN	extern
#endif

#endif

#if !defined(GBASIC_STATIC_INLINE)
#define GBASIC_STATIC_INLINE	static __inline__
#endif

#if !defined(GBASIC_EXTERN_INLINE)
#define GBASIC_EXTERN_INLINE	extern __inline__
#endif

/**
 * Basic retain, release, and autorelease operations.
 */
#ifndef RETAIN
#define RETAIN(object)		[object retain]
#endif

#ifndef RELEASE
#define RELEASE(object)		[object release]
#endif

#ifndef AUTORELEASE
#define AUTORELEASE(object)	[object autorelease]
#endif

/**
 * Tested retain, release, and autorelease operations - only invoke the
 * objective-c method if the receiver is not nil.
 */
#ifndef TEST_RETAIN
#define TEST_RETAIN(object)	(object != nil? [object retain]: nil)
#endif

#ifndef TEST_RELEASE
#define TEST_RELEASE(object)	({ if (object) [object release]; })
#endif

#ifndef TEST_AUTORELEASE
#define TEST_AUTORELEASE(object)	({ if (object) [object autorelease]; })
#endif

/**
 * ASSIGN(object, value) assigns the value to the object with
 * appropriate retain and release operations.
 */
#ifndef ASSIGN
#define ASSIGN(object, value)	\
do {\
    id __value = (id)(value);\
    id __object = (id)(object);\
    if (__value != __object) {\
	if (__value != nil) {\
	    [__value retain];\
	}\
	(id)(object) = __value;\
	if (__object != nil) {\
	    [__object release];\
	}\
    }\
} while (0)
#endif

/**
 * ASSIGNCOPY(object, value) assignes a copy of the value to the object with
 * and release operations.
 */
#ifndef ASSIGNCOPY
#define ASSIGNCOPY(object, value)	\
do {\
    id __value = (value);\
    id __object = (id)(object);\
    if (__value != __object) {\
	if (__value != nil) {\
	    __value = [__value copy];\
	}\
	(id)(object) = __value;\
	if (__object != nil) {\
	    [__object release];\
	}\
    }\
} while (0)
#endif

#ifndef ASSIGNCOPYZONE
#define ASSIGNCOPYZONE(object, value, zone)	\
do {\
    id __value = (value);\
    id __object = (id)(object);\
    if (__value != __object) {\
	if (__value != nil) {\
	    __value = [__value copyWithZone: (zone)];\
	}\
	(id)(object) = __value;\
	if (__object != nil) {\
	    [__object release];\
	}\
    }\
} while (0)
#endif

/**
 * DESTROY() is a release operation which also sets the variable to be
 * a nil pointer for tidyness - we can't accidentally use a DESTROYED
 * object later.  It also makes sure to set the variable to nil before
 * releasing the object - to avoid side-effects of the release trying
 * to reference the object being released through the variable.
 */
#ifndef DESTROY
#define DESTROY(object)	\
do {\
    if (object) {\
	id __o = (object);\
	(object) = nil;\
	[__o release];\
    }\
} while (0)
#endif

#import <Foundation/NSObjCRuntime.h>

GBASIC_EXTERN BOOL GGetSystemVersion(unsigned long *);

GBASIC_EXTERN IMP GReplaceInstanceMethodForSelector(Class aClass, SEL oldSelector, SEL newSelector);
GBASIC_EXTERN IMP GReplaceClassMethodForSelector(Class aClass, SEL oldSelector, SEL newSelector);

GBASIC_EXTERN void GInitializeGBasic(BOOL initializeAllCategories);

#import <objc/objc-class.h>

#ifndef CLS_ISCLASS
#define CLS_ISCLASS(CLASSPOINTER) \
    ((((struct objc_class*)(CLASSPOINTER))->info) & CLS_CLASS)
#endif

GBASIC_STATIC_INLINE BOOL GObjCIsInstance(id object)
{
    return (object && CLS_ISCLASS(object->isa));
}

GBASIC_STATIC_INLINE BOOL GObjCIsClass(Class aClass)
{
    return (aClass && CLS_ISCLASS(aClass));
}

GBASIC_STATIC_INLINE unsigned GObjCHash(id object)
{
#if defined(__ppc__)
    /* 16 byte alignment */
    return (((unsigned)object >> 4) | ((unsigned)object << (32-4)));
#else
    /* 8 byte alignment */
    return (((unsigned)object >> 3) | ((unsigned)object << (32-3)));
#endif
}

#endif /* __GBASIC_GOBJCRUNTIME_H__ */
