#ifndef __GBASIC_NSDATE_EXTENSIONS_H__
#define __GBASIC_NSDATE_EXTENSIONS_H__
/**********************************************************************
  GBasic/NSDate-Extensions.h
**********************************************************************/
/*-
 * Copyright (c) 2004  Goichi Hirakawa  <gooichi AT gyazsquare DOT com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#import <Foundation/NSDate.h>
#import <Foundation/NSUserDefaults.h>

//-----------------------------------------------------------------------------
//  NSDate (GBasic) interface
//-----------------------------------------------------------------------------
@interface NSDate (GBasic)

- (int)milliSecond;
- (int)microSecond;

- (BOOL)isBeforeDate: (NSDate*)date;
- (BOOL)isAfterDate: (NSDate*)date;

@end

typedef short GDateForm;
enum {
    GShortDate		= 0,
    GLongDate		= 1,
    GAbbrevDate		= 2
};

enum {
    GRelativeDateMask	= 4	/* use a relative date if available */
};

//-----------------------------------------------------------------------------
//  NSDate (DateAndTimeDescriptions) interface
//-----------------------------------------------------------------------------
@interface NSDate (DateAndTimeDescriptions)
- (NSString*)dateDescriptionWithDateForm: (GDateForm)longFlag timeZone: (NSTimeZone*)aTimeZone locale: (NSDictionary*)locale;
- (NSString*)timeDescriptionWithSeconds: (BOOL)wantSeconds timeZone: (NSTimeZone*)aTimeZone locale: (NSDictionary*)locale;
- (NSString*)dateTimeDescriptionWithDateForm: (GDateForm)longFlag seconds: (BOOL)wantSeconds separator: (NSString*)separator timeZone: (NSTimeZone*)aTimeZone locale: (NSDictionary*)locale;
@end

//-----------------------------------------------------------------------------
//  NSUserDefaults (DateAndTimeFormats) interface
//-----------------------------------------------------------------------------
@interface NSUserDefaults (DateAndTimeFormats)
- (NSString*)dateFormatWithDateForm: (GDateForm)dateForm;
- (NSString*)timeFormatWithSeconds: (BOOL)wantSeconds;
@end

#endif /* __GBASIC_NSDATE_EXTENSIONS_H__ */
