#ifndef __GBASIC_NSDICTIONARY_EXTENSIONS_H__
#define __GBASIC_NSDICTIONARY_EXTENSIONS_H__
/**********************************************************************
  GBasic/NSDictionary-Extensions.h
**********************************************************************/
/*-
 * Copyright (c) 2004  Goichi Hirakawa  <gooichi AT gyazsquare DOT com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#import <Foundation/NSDictionary.h>

//-----------------------------------------------------------------------------
//  NSDictionary (GBasicExtendedDictionary) interface
//-----------------------------------------------------------------------------
@interface NSDictionary (GBasicExtendedDictionary)

- (id)objectOfClass: (Class)class forKey: (id)key;
- (id)codingObjectForKey: (id)key;

- (short)shortForKey: (id)key; /* defaultValue = 0 */
- (short)shortForKey: (id)key defaultValue: (short)defaultValue;

- (unsigned short)unsignedShortForKey: (id)key; /* defaultValue = 0 */
- (unsigned short)unsignedShortForKey: (id)key defaultValue: (unsigned short)defaultValue;

- (int)intForKey: (id)key; /* defaultValue = 0 */
- (int)intForKey: (id)key defaultValue: (int)defaultValue;

- (unsigned)unsignedIntForKey: (id)key; /* defaultValue = 0 */
- (unsigned)unsignedIntForKey: (id)key defaultValue: (unsigned)defaultValue;

- (float)floatForKey: (id)key; /* defaultValue = 0.0 */
- (float)floatForKey: (id)key  defaultValue: (float)defaultValue;

- (double)doubleForKey: (id)key; /* defaultValue = 0.0 */
- (double)doubleForKey: (id)key defaultValue: (double)defaultValue;

- (BOOL)boolForKey: (id)key; /* defaultValue = NO */
- (BOOL)boolForKey: (id)key defaultValue: (BOOL)defaultValue;

- (BOOL)isEqualToDictionary: (NSDictionary*)otherDictionary exceptForKey: (id)key;
- (BOOL)isEqualToDictionary: (NSDictionary*)otherDictionary exceptForKeys: (NSArray*)keys;

@end

//-----------------------------------------------------------------------------
//  NSMutableDictionary (GBasicExtendedMutableDictionary) interface
//-----------------------------------------------------------------------------
@interface NSMutableDictionary (GBasicExtendedMutableDictionary)

- (void)setCodingObject: (id)object forKey: (id)key;

- (void)setShort: (short)val forKey: (id)key;
- (void)setUnsignedShort: (unsigned short)val forKey: (id)key;
- (void)setInt: (int)val forKey: (id)key;
- (void)setUnsignedInt: (unsigned int)val forKey: (id)key;
- (void)setFloat: (float)val forKey: (id)key;
- (void)setDouble: (double)val forKey: (id)key;
- (void)setBool: (BOOL)val forKey: (id)key;

@end

#endif /* __GBASIC_NSDICTIONARY_EXTENSIONS_H__ */
