#ifndef __GBASIC_NSFILEMANAGER_EXTENSIONS_H__
#define __GBASIC_NSFILEMANAGER_EXTENSIONS_H__
/**********************************************************************
  GBasic/NSFileManager-Extensions.h
**********************************************************************/
/*-
 * Copyright (c) 2004  Goichi Hirakawa  <gooichi AT gyazsquare DOT com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#import <Foundation/NSFileManager.h>

//-----------------------------------------------------------------------------
//  NSFileManager (GBasic) interface
//-----------------------------------------------------------------------------
@interface NSFileManager (GBasic)

- (BOOL)fileExistsAtPath: (NSString*)path isDirectory: (BOOL*)isDirectory traverseLink: (BOOL)yorn;
- (BOOL)isHiddenFileAtPath: (NSString*)path;

/* simple 'rename' */
- (BOOL)renamePath: (NSString*)src toPath: (NSString*)dest handler: handler;

- (BOOL)createFileExclusivelyAtPath: (NSString*)path;

- (NSString*)createTemporaryFileUsingName: (NSString*)name;
- (NSString*)createTemporaryFileUsingPath: (NSString*)path;
- (NSString*)createTemporaryFileUsingTemplate: (NSString*)template suffixLength: (unsigned)suffixLength;

- (NSString*)createNumberedFileUsingPath: (NSString*)path;
- (NSString*)createNumberedFileUsingPath: (NSString*)path numberFormat: (NSString*)numberFormat startNumber: (int)startNumber suffixLength: (unsigned)suffixLength;

@end

#if defined(MACOSX)

struct FSRef;
struct FSSpec;

//-----------------------------------------------------------------------------
//  NSFileManager (GBasicCarbonExtensions) interface
//-----------------------------------------------------------------------------
@interface NSFileManager (GBasicCarbonExtensions)

- (NSString*)pathWithFSRef: (const struct FSRef*)ref;

- (BOOL)getFSRef: (struct FSRef*)ref withPath: (NSString*)path;
- (BOOL)getFSRef: (FSRef*)ref isDirectory: (BOOL*)isDirectory withPath: (NSString*)path;

- (BOOL)getFSSpec: (struct FSSpec*)spec withPath: (NSString*)path;
- (BOOL)getFSSpec: (FSSpec*)spec isDirectory: (BOOL*)isDirectory withPath: (NSString*)path;

- (BOOL)getFinderInfo: (NSData**)finderInfo extendedFinderInfo: (NSData**)extendedFinderInfo isDirectory: (BOOL*)isDirectory withPath: (NSString*)path;
- (BOOL)setFinderInfo: (NSData*)finderInfo extendedFinderInfo: (NSData*)extendedFinderInfo withPath: (NSString*)path;

- (BOOL)getResourceFork: (NSData**)data withPath: (NSString*)path;
- (BOOL)setResourceFork: (NSData*)data withPath: (NSString*)path;

@end
#endif /* defined(MACOSX) */

#endif /* __GBASIC_NSFILEMANAGER_EXTENSIONS_H__ */
