#ifndef __GBASIC_SPIN_LOCK_H__
#define __GBASIC_SPIN_LOCK_H__
/**********************************************************************
  GBasic/SpinLock.h
**********************************************************************/
/*-
 * Copyright (c) 2004  Goichi Hirakawa  <gooichi AT gyazsquare DOT com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>

__BEGIN_DECLS

typedef uint32_t GSpinLock_t;

static __inline__ void GSpinLockInit(GSpinLock_t *lockp)
{
    *lockp = 0;
}

static __inline__ void GSpinLockDestroy(GSpinLock_t *lockp)
{
    /* empty */
}

#if defined(__MACH__)

extern int _spin_lock_try(uint32_t *lockp);
extern void _spin_lock(uint32_t *lockp);
extern void _spin_unlock(uint32_t *lockp);

static __inline__ int GSpinLockTry(GSpinLock_t *lockp)
{
    return _spin_lock_try(lockp);
}

static __inline__ void GSpinLock(GSpinLock_t *lockp)
{
    _spin_lock(lockp);
}

static __inline__ void GSpinUnlock(GSpinLock_t *lockp)
{
    _spin_unlock(lockp);
}

#else /* !defined(__MACH__) */

#warning GBasic spin locks not defined for this platform

#define GSpinLockTry(A)	do { return 0; } while (0)
#define GSpinLock(A)	do {} while (0)
#define GSpinUnlock(A)	do {} while (0)

#endif /* !defined(__MACH__) */

__END_DECLS

#endif /* __GBASIC_SPIN_LOCK_H__ */
